/* Copyright (c) 2008 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is confidential property of 
 * Nordic Semiconductor. The use, copying, transfer or disclosure 
 * of such information is prohibited except by express written
 * agreement with Nordic Semiconductor.
 */

 /** @file
 *
 * @author Torbjrn vrebekk
 *
 */
 
#include <Nordic\reg24lu1.h>
#include <stdint.h>
#include <stdbool.h>
#include <string.h>

#include "hal_nrf.h"
#include "hal_flash.h"
#include "cklf.h"
#include "cpu.h"
#include "usb_api.h"
#include "hal_uart.h"
#include "usb_uart.h"
#include "radio.h"

void mainloop_uart();
void mainloop_echo();
void mainloop_radio();

#define BUFSIZE 32
xdata uint8_t radioBuf[32], radioInBuf[32];
xdata volatile uint8_t usbBuffer[BUFSIZE], usbBufferCount = 0, timeToSend = 0;


void delay()
{
  xdata uint16_t delCnt = 2000;
  while(delCnt--);
}
uint8_t usb_uart_getchar()
{
  //while( !usb_uart_buffer_status( &uart_tx_buffer ));
  return usb_uart_buffer_get( &uart_tx_buffer );
}
void usb_uart_putchar(uint8_t byte)
{
  usbBuffer[usbBufferCount++] = byte;
}
uint8_t usb_uart_has_data()
{
  return uart_tx_buffer.size > 0;
}
void usb_uart_init()
{
  usb_uart_buffer_clear( &uart_tx_buffer );
}
void put_string(char *string)
{
  while(*string)usb_uart_putchar(*string++);
}  

void main(void)
{
  usbBuffer[8] = 0;
  // Set up GPIO                          
  P0DIR = 0x00;
  P0 = 0x00;                        

  usb_init();                                  

  usb_uart_init();
  
  EA=1;                    // Enable global IRQ
                            
  usb_wait_for_configuration();               // Wait until USB enumerated

  mainloop_radio();
  //mainloop_uart();
  //mainloop_echo();

}

void timerIrq() interrupt INTERRUPT_T0
{
  TH0 = 0xD7;       // 1 ms timer period when in mode 0 (13 bit)
  if( usbBufferCount ) timeToSend = 1;
}

void mainloop_echo()
{
  uint8_t tmp;                            
  TMOD &= 0xF0;
  TR0 = 1;
  ET0 = 1;
  while(true)
  {
    
    if( usb_uart_has_data() )   // Echo
    {
      tmp = usb_uart_getchar();
      usb_uart_putchar(tmp);
    }
    if( timeToSend )
    {
      hal_usb_send_data(0x82, usbBuffer, usbBufferCount);
      usbBufferCount = 0;
      timeToSend = 0;
    }
    delay();
  }
}
void mainloop_uart()
{
  P0DIR = 0x02;
  TMOD &= 0xf0;
  TR0 = 1;
  ET0 = 1;
  hal_uart_init(UART_BAUD_19K2);
  while(true)
  {
    if( usb_uart_has_data() )
      hal_uart_putchar(usb_uart_getchar());
    if( hal_uart_chars_available() )
      usb_uart_putchar(hal_uart_getchar());
    if( timeToSend )
    {
      hal_usb_send_data(0x82, usbBuffer, usbBufferCount);
      usbBufferCount = 0;
      timeToSend = 0;
    }
  }
}
void mainloop_radio()
{
  uint8_t loopCounter = 0, tmp, i, inchar;
  radio_init();
  TMOD &= 0xF0;
  TR0 = 1;
  ET0 = 1;
  while(true)
  {
    if( usb_uart_has_data() )   // Echo
    {
      tmp = 0;
      while(usb_uart_has_data() )
      {
        inchar = usb_uart_getchar();
        radioBuf[tmp++] = inchar;
        usb_uart_putchar(inchar); 
        if( tmp >= 32 ) break;
      }
      set_radio_mode(HAL_NRF_PTX);
      radio_send_packet(radioBuf,tmp); 
      while( get_radio_status() == RF_BUSY );
      P01 ^= 1; 
      set_radio_mode(HAL_NRF_PRX);          
    }

    if( get_radio_status() == HAL_NRF_RX_DR )
    {      
      tmp = radio_read_data(radioInBuf);
      for( i = 0; i < tmp; i++ )
        usb_uart_putchar(radioInBuf[i]); 
    }  
    // Blink light
    if( (loopCounter & 0x3f) == 0)
    {
      P00 = !P00;
    }
    delay();
    if( timeToSend )
    {
      hal_usb_send_data(0x82, usbBuffer, usbBufferCount);
      usbBufferCount = 0;
      timeToSend = 0;
    }
    loopCounter++;
  }
}