#include "radio.h"
#include "hal_nrf.h"
#include "hal_nrf_reg.h"
#include "hal_nrf_hw.h"

uint8_t theAddress[5] = {0x22,0x33,0x44,0x55,0x01};
uint8_t radio_status;
void radio_init(void)
{
  RFCKEN = 1;                                     // enable L01 clock
  RFCTL = 0x10;    
	hal_nrf_activate_features();										// Enable hidden features. (We need dynamic payload)
	hal_nrf_enable_ack_payload(true);								// Enable dynamic ack
	hal_nrf_enable_dynamic_payload(true);						// Activate dynamic payload length
	hal_nrf_setup_dynamic_payload(0xFF);						// All pipes uses dynamic ack.

  hal_nrf_close_pipe(HAL_NRF_ALL);                // first close all radio pipes...
                                                  // Pipe 0 and 1 open by default.
  hal_nrf_open_pipe(HAL_NRF_PIPE0, true);         // then open pipe0, with autoack

  hal_nrf_set_crc_mode(HAL_NRF_CRC_16BIT);        // operates in 16bits CRC mode
  hal_nrf_set_auto_retr(3, 250);                  // 250 s delay, 3 retransmits

  hal_nrf_set_address_width(HAL_NRF_AW_5BYTES);   // 5 bytes address width
  hal_nrf_set_address(HAL_NRF_TX, theAddress);       // set device's addresses
  hal_nrf_set_address(HAL_NRF_PIPE0, theAddress);    // Pipe0 used for auto ACK

  hal_nrf_set_operation_mode(HAL_NRF_PRX);      // enter TX mode

  
  hal_nrf_set_rf_channel(40);                     // operating on static channel: 40 (2440MHz)
  hal_nrf_set_power_mode(HAL_NRF_PWR_UP);         // power up device
  RFCE = 1;
  RF = 1;                                         // enable RF interrupt
  radio_status = RF_IDLE;                         // radio now ready, i.e. RF_IDLE
}
void set_radio_mode(uint8_t newMode)
{
  if( newMode == HAL_NRF_PRX )
  {
    RFCE = 0;
    hal_nrf_set_operation_mode(HAL_NRF_PRX);
    RFCE = 1;
  }
  else if( newMode == HAL_NRF_PTX )
  {
    RFCE = 0;
    hal_nrf_set_operation_mode(HAL_NRF_PTX);
  }
  radio_status = RF_IDLE;
}
void radio_irq(void) interrupt RF_READY_INT_VECT  
{
  switch(hal_nrf_get_clear_irq_flags())
  {
    case (1<<HAL_NRF_MAX_RT):                     // max retries reach LED2_flash
      hal_nrf_flush_tx();                         // simply flush tx fifo, avoid fifo jamming
      radio_status = HAL_NRF_MAX_RT;
      break;
    
    case (1<<HAL_NRF_TX_DS):                      // packet sent, LED1_flash      
      radio_status = HAL_NRF_TX_DS;
      break;
    
    case (1<<HAL_NRF_RX_DR):                      // packet received, LED1_flash
      //hal_nrf_read_rx_payload(packet);
      
      radio_status = HAL_NRF_RX_DR;
      break;
      
    default:
      break;    
  }
}
void radio_send_packet(uint8_t *buf, uint8_t len) //_with_counter
{
	
  hal_nrf_write_tx_payload(buf, len);// load message into radio

	CE_PULSE();                                     	// send packet
  radio_status = RF_BUSY;                         	// trans. in progress; RF_BUSY
}
uint8_t get_radio_status()
{
  return radio_status;
}

uint8_t radio_read_data(uint8_t *rx_data)
{
  uint8_t length = hal_nrf_read_rx_payload(rx_data); // Get received data,
                                                        // and bytes received   
  if (hal_nrf_get_rx_fifo_status() == 0x01)       // Fifo is empty?	
  {
    radio_status = RF_IDLE;
  }
  return length;
}