#include "usb_uart.h"
#include "radio.h"

static data uint8_t tmp_usb_buf[8];

data tRingBuffer uart_tx_buffer;
xdata void(*lineCodeCallback)(hal_usb_uart_line_coding_t *settings) = NULL;

xdata uint32_t usb_uart_baudrate_swapped = 115200;
hal_usb_uart_line_coding_t xdata usb_uart_line_coding = {
	12714240,		
	0,		
	0,		
	8			
};
volatile bit uart_DTR;			
volatile bit uart_RTS;

uint32_t swap_bytes(uint32_t arg)
{
  uint32_t returnValue = (arg >> 24) & 0x000000ff;
  returnValue |= (arg >> 8) &  0x0000ff00;
  returnValue |= (arg << 8) &  0x00ff0000;
  returnValue |= (arg << 24) & 0xff000000;
  return returnValue;
}
hal_usb_dev_req_resp_t usb_uart_device_req_proc(hal_usb_device_req* req, uint8_t** data_ptr, uint16_t* size, hal_usb_dev_req_resp_t *resp)
{
  uint8_t i;
  bool retval = false;
  *resp = STALL; 
  /*if(req->bRequest == USB_REQ_GET_DESCRIPTOR )
  {
    switch( MSB(req->wValue) )
    {
      case  USB_CLASS_DESCRIPTOR_REPORT:
        break;
      case USB_CLASS_DESCRIPTOR_HID:
        break;
      default:
        break;
    }
  } */
  if( ( req->bmRequestType & 0x20 ) == 0x20 ) // This is a class specific request D5..6: Type Class(value 1)
  { 
    switch( req->bRequest )
    {
      case 0:   // Send encapsulated command.
		    *resp = EMPTY_RESPONSE;
        retval = true;    
        break;
      case 1:   // Get encapsulated command.
		    *resp = EMPTY_RESPONSE;
        retval = true;   
        break;

      case 32:  // Set line coding.
      	if (   (req->bmRequestType == 33) && (req->wValueMsb == 0 && req->wValueLsb == 0) && (req->wLength == sizeof(hal_usb_uart_line_coding_t)) )
      	{
#warning "Must comment in misc_data in hal_usb.h!"
      		for( i = 0; i < sizeof(hal_usb_uart_line_coding_t); i++ )
            *(((uint8_t*)(&usb_uart_line_coding))) = ((uint8_t*)req->misc_data)[i];
          usb_uart_baudrate_swapped = swap_bytes(usb_uart_line_coding.baudrate);
          if( lineCodeCallback ) lineCodeCallback(&usb_uart_line_coding);
          *resp = EMPTY_RESPONSE;
          retval = true;    
      	}
        break;

      case 33:  // Get line coding.
        #warning 161 konstant
        if ( (req->bmRequestType == 161) && (req->wValueMsb == 0 && req->wValueLsb == 0) && (req->wLength == sizeof(hal_usb_uart_line_coding_t)) ) 
      	{
          *data_ptr = (uint8_t *)(&usb_uart_line_coding);  
          *size = sizeof(hal_usb_uart_line_coding_t);
          *resp = DATA;
          retval = true;
        }    
        break;
      case 34:  // Set control line state.
       	if (   (req->bmRequestType == 33) && (req->wLength == 0) )
        {
        	usb_uart_set_line_state( (uint8_t)req->wValueLsb & (UART_RTS | UART_DTR ) );
          *resp = EMPTY_RESPONSE;
          retval = true;
        }
        break;
      case 35:  // Send break.
        break;
      default:
        break;
    }
  }
  return retval;
}

void usb_uart_set_line_state(uint8_t state)
{
	//P04 = uart_DTR = ((state & UART_DTR) != 0);
	//P05 = uart_RTS = ((state & UART_RTS) != 0);
}
void usb_uart_set_line_state_change_callback(void (*callback)(hal_usb_uart_line_coding_t *settings))
{
  lineCodeCallback = callback;
}

void usb_uart_buffer_clear( tRingBuffer *buf )
{
	buf->pos_in = buf->pos_out = buf->size = 0;
}

uint8_t usb_uart_buffer_put( tRingBuffer *buf, uint8_t a_Byte )
{
	if( buf->size < UART_BUFFER_SIZE )
	{
    buf->buffer[buf->pos_out] = a_Byte;
    buf->pos_out++;
    buf->pos_out &= UART_BUFFER_MODULO;
    buf->size++;
    return 1;
  }
  else return 0;
}
uint8_t usb_uart_buffer_put_array( tRingBuffer *buf, uint8_t *array, uint8_t size )
{
  if( (buf->size + size) <= UART_BUFFER_SIZE )
  {
    while( size-- )
    {
      buf->buffer[buf->pos_out] = *array++;    //61
      buf->pos_out = (buf->pos_out + 1 ) & UART_BUFFER_MODULO;   //19
      buf->size++; //11  
    }
    return 1;
  }    
  else
  {
    while( buf->size < UART_BUFFER_SIZE )
    {
      buf->buffer[buf->pos_out] = *array++;
      buf->pos_out = (buf->pos_out + 1 ) & UART_BUFFER_MODULO; 
      buf->size++;
    }
    return 0;
  }
}
uint8_t usb_uart_buffer_get( tRingBuffer *buf )
{
  uint8_t retVal = 0;
  if( buf->size > 0 )
  {
    retVal = buf->buffer[buf->pos_in];
    buf->pos_in++;
    buf->pos_in &= UART_BUFFER_MODULO;
    buf->size--;
  }
  return retVal;
}
uint8_t usb_uart_buffer_status( tRingBuffer *buf )
{
  if( buf->size == 0 ) return BUFFER_EMPTY;
  if( buf->size == UART_BUFFER_SIZE ) return BUFFER_FULL;
  else return BUFFER_HAS_DATA;
}
      
    
    
		
